/*******************************************************************************
**
**	File:		Cmsg.h
**
**	Description:	Functions for create Cyradis BMessages.
**
**	© Copyright 1997, Cyradis Corporation
**
*******************************************************************************/

#ifndef _CMSG_H_
#define _CMSG_H_

#ifndef _CYRADIS_MSG_H_
#include "Cyradis_msg.h"
#endif


BMessenger *CreateMessenger(void);

BMessage *CreateScene(const char *);
void SetId(BMessage *, int32);
BMessage *CreateRender(int32 height, int32 width, bool antialiasing, int32 linestart, int32 lineend);
BMessage *CreateCamera(double PosX,double PosY, double PosZ, double LookX, double LookY, double LookZ, double Roll, double Lens);

void SetObjectColor(BMessage *obj,rgb_color &);
void SetObjectAmbient(BMessage *obj,double ambient);
void SetObjectDiffuse(BMessage *obj,double diffuse);
void SetObjectReflection(BMessage *obj,double reflection);
void SetObjectTransparency(BMessage *obj,double transparency);
void SetObjectIor(BMessage *obj,double ior);
void SetObjectSpecular(BMessage *obj,double specular);
void SetObjectSpecularSize(BMessage *obj,double specsize);

BMessage *CreateSphere(double X, double Y, double Z, double Radius);
BMessage *CreatePlane(double A, double B, double C, double D);
			// Ax + By + Cz = D
BMessage *CreateDisc(double x1,double y1,double z1, double x2,double y2,double z2, double radiusmin, double radiusmax);
			// 3 first parameters are the center of the disc, the others are the coordinates of the normal vector of the disc.
BMessage *CreateCone(double X1, double Y1, double Z1, double Radius1, double X2, double Y2, double Z2, double Radius2);
BMessage *CreateCylinder(double X1, double Y1, double Z1, double X2, double Y2, double Z2, double Radius);

void SetConeOpen(BMessage *cone);
void SetCylinderOpen(BMessage *cylinder);

BMessage *CreateQuadric(double A, double B, double C, double D, double E, double F, double G, double H, double I, double J);
			// Ax2 + By2 + Cz2 + Dxy + Exz + Fyz + Gx + Hy + Iz + J = 0

BMessage *CreateTriangle(double ax,double ay,double az,double bx,double by,double bz,double cx,double cy,double cz);
BMessage *CreateBox(double ax,double ay,double az,double bx,double by,double bz);

BMessage *CreateDifference(const BMessage *object1, const BMessage *object2);
BMessage *CreateUnion(const BMessage *object1, const BMessage *object2);
BMessage *CreateMerge(const BMessage *object1, const BMessage *object2);
BMessage *CreateClip(const BMessage *object1, const BMessage *object2);
BMessage *CreateIntersection(const BMessage *object1, const BMessage *object2);

void AddBoundedObject(BMessage *object, const BMessage *bounded_by);


BMessage *CreateLight(double X, double Y, double Z, rgb_color &color);
BMessage *CreateSpot(double PosX,double PosY, double PosZ, double LookX, double LookY, double LookZ, double RadiusMin, double RadiusMax, rgb_color &color);
BMessage *CreateSun(double X, double Y, double Z, rgb_color &color);
void SetRange(BMessage *message,double min,double max);


// texture BMessage
BMessage *CreateBump(void);
BMessage *CreateChecker(int16 red1, int16 green1, int16 blue1, int16 red2, int16 green2, int16 blue2);
BMessage *CreateMapping(const char *filename, bool once, int8 type);
void AddMappingFilter(BMessage *mapping, int16 red, int16 green, int16 blue);
BMessage *CreateNoiseBump(void);
void AddNoiseBumpLevel(BMessage *noise, float level);
void AddNoiseOctave(BMessage *noise, int8 octave);
BMessage *CreateNoiseColor(void);
void AddColorMap(BMessage *colormap,double limit, int16 red, int16 green, int16 blue);
void AddTexture(BMessage *dest, const BMessage *texture);
		// dest must an object

BMessage *CreateScale(double X, double Y, double Z);
BMessage *CreateRotate(double X, double Y, double Z);
BMessage *CreateTranslate(double X, double Y, double Z);
void AddTransformation(BMessage *dest, const BMessage *transf);
		// Dest can be either an object or a texture;


// not implemented yet ...
BMessage *CreateQuartic(double a00, double a01, double a02, double a03, double a04, double a05, double a06, double a07, double a08, double a09, double a10, double a11, double a12, double a13, double a14, double a15, double a16, double a17, double a18, double a19, double a20, double a21, double a22, double a23, double a24, double a25, double a26, double a27, double a28, double a29, double a30, double a31, double a32, double a33, double a34);
			//   a00x^4 + a01x^3y + a02x^3z + a03x^3 + a04x^2y^2
			// + a05x^2yz + a06x^2y + a07x^2z^2 + a08x^2z + a09x^2
			// + a10xy^3 + a11xy^2z + a12xy^2 + a13xyz^2 + a14xyz
			// + a15xy + a16xz^3 + a17xz^2 + a18xz + a19x
			// + a20y^4 + a21y^3z + a22y^3 + a23y^2z^2 + a24y^2z
			// + a25y^2 + a26yz^3 + a27yz^2 + a28yz + a29y
			// + a30z^4 + a31z^3 + a32z^2 + a33z + a34





void SendCyradisMessage(BMessage, int32 scene_number);


#endif